import { jsx as _jsx } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import * as React from "react";
import { animate, motion, useMotionValue, } from "framer-motion";
import { MoveFocusInside } from 'react-focus-lock';
const pageStyle = {
    width: "100%",
    height: "100%",
    display: "inline-block",
    flex: "none",
};
const Slider = ({ x, i, onDragEnd, children }) => (_jsx(motion.div, { className: "Page", style: {
        ...pageStyle,
        x,
        left: `${i * 100}%`,
        right: `${i * 100}%`,
    }, css: {
        overflowY: 'auto',
    }, drag: "x", dragElastic: 0.3, onDragEnd: onDragEnd, dragDirectionLock: true, children: children }));
const containerStyle = {
    position: "relative",
    width: "100%",
    height: "100%",
    overflowX: "hidden",
    display: "flex",
};
const transition = {
    type: "spring",
    bounce: 0,
};
const Contaier = React.forwardRef((props, ref) => (_jsx("div", { className: "DragContaier", ref: ref, style: containerStyle, children: props.children })));
/**轮播台; 支持嵌套多层的 Pager 轮播台，可单独控制。
 * 改用 GesturePage 支持嵌套和内部链接点击。
 * */
export const CarouselPage = ({ children, loop = false, value = 0, onRequestChange, onNestDragEnd, }) => {
    //轮播台自身：
    const containerRef = React.useRef(null);
    const x = useMotionValue(0);
    /* 用外部注入替代：value，onRequestChange，状态管理的控制外移上一级组件！
        const [index, setIndex] = React.useState(0)     回调触发：切换到哪一个儿子去了
    * */
    //窗口偏移模式：轮播台的儿子都是相等的窗口clientWidth宽度的：X-水平横轴飘动。
    const calculateNewX = React.useCallback(() => -value * (containerRef.current?.clientWidth || 0), [value]);
    const handleEndDrag = (e, dragProps) => {
        const clientWidth = containerRef.current?.clientWidth || 0;
        const { offset, velocity } = dragProps;
        const xDrag = Math.abs(velocity.x) > Math.abs(velocity.y);
        if (!xDrag) {
            return animate(x, calculateNewX(), transition);
        }
        if (offset.x > clientWidth / 4) {
            if (handlePrev())
                onNestDragEnd(e, dragProps);
        }
        else if (offset.x < -clientWidth / 4) {
            if (handleNext())
                onNestDragEnd(e, dragProps);
        }
        else {
            animate(x, calculateNewX(), transition);
        }
    };
    const childrens = React.Children.toArray(children);
    //原地踏步吗：return true:没有拉动本轮播台。
    const handleNext = () => {
        const toIndex = (value + 1) >= childrens.length ? (loop ? 0 : value) : (value + 1);
        if (toIndex === value) {
            animate(x, calculateNewX(), transition);
            return true;
        }
        onRequestChange(toIndex);
        return false;
    };
    //原地踏步吗：true。
    const handlePrev = () => {
        const toIndex = (value - 1 < 0) ? (loop ? childrens.length - 1 : 0) : (value - 1);
        if (toIndex === value) {
            animate(x, calculateNewX(), transition);
            return true;
        }
        onRequestChange(toIndex);
        return false;
    };
    React.useEffect(() => {
        const controls = animate(x, calculateNewX(), transition);
        return controls.stop;
    }, [calculateNewX, value, x]);
    return (_jsx(Contaier, { ref: containerRef, children: childrens.map((child, i) => _jsx(Slider, { onDragEnd: handleEndDrag, x: x, i: i, children: _jsx(MoveFocusInside, { children: child }) }, i)) }));
};
//# sourceMappingURL=CarouselPage.js.map