import { jsx as _jsx } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import PropTypes from "prop-types";
export function getEmbedStyles(width, height) {
    const percentage = (height / width) * 100;
    return css `
    position: relative;
    display: block;
    width: 100%;
    padding: 0;
    & > * {
      position: absolute;
      top: 0;
      bottom: 0;
      left: 0;
      width: 100%;
      height: 100%;
      border: 0;
    }
    ::before {
      display: block;
      content: "";
      padding-top: ${percentage}%;
    }
  `;
}
/**
 * The Embed component maintains the provided aspect ratio determined
 * by the width and height props. It's useful for avoiding content shifting on image
 * loading.
 */
export const Embed = ({ width = 4, height = 3, ...other }) => {
    return _jsx("div", { css: getEmbedStyles(width, height), ...other });
};
Embed.propTypes = {
    width: PropTypes.number,
    height: PropTypes.number
};
//# sourceMappingURL=Embed.js.map