import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { useSprings, animated } from '@react-spring/web';
// @ts-ignore
import { useGesture } from '@use-gesture/react';
import { useMeasure } from "./Hooks/use-measure";
import { usePrevious } from "./Hooks/previous";
/*任意内容 轮播台 类似于 CarouselPage ；
文档例子：https://codesandbox.io/s/github/pmndrs/use-gesture/tree/main/demo/src/sandboxes/draggable-list
* */
export const GesturePage = React.forwardRef(({ children, id, value: index, onRequestChange, lazyLoad = false, onSetLazy, onNestDragEnd, style, paneStyle, noBind = false, ...other }, ref) => {
    const indexRef = React.useRef(index);
    const containerRef = React.useRef(null); //绑定 拉拽的第一层Div;
    const { bounds } = useMeasure(containerRef);
    const { width, } = bounds;
    //已经加载的部分： 多个儿子，不一定都加载mount完毕; 轮播切换不会卸载。 [index]默认只能加载一个;
    const [loaded, setLoaded] = React.useState(() => new Set(onSetLazy ? onSetLazy(index) : [index]));
    //当前已经加载的儿子映射DOM;
    const childrenRefs = React.useRef(new Map());
    const previousIndex = usePrevious(index);
    const shouldFocusRef = React.useRef(index || null);
    React.useEffect(() => {
        if (typeof previousIndex === "number" && previousIndex !== index) {
            shouldFocusRef.current = index;
        }
        else {
            shouldFocusRef.current = null;
        }
    }, [previousIndex, index]);
    //直接定位滚到某个儿子吗？
    function focusByIndex(i) {
        const el = childrenRefs.current.get(i);
        if (el) {
            el.focus();
        }
    }
    // expose an imperative focus function which focuses the currently active index
    //上级组件可能使用本接口直接操作 focusByIndex切换到某一个儿子， 跳过轮播滑动:也是可能切换轮播台的。
    React.useImperativeHandle(ref, () => ({
        focus: (i) => {
            focusByIndex(i || index);
        }
    }));
    const renderableChildren = children.filter(child => child !== null);
    const childCount = renderableChildren.length;
    function isValidNextIndex(index) {
        return index >= 0 && index <= (childCount - 1);
    }
    /** 跟踪设置应该加载的儿子;
     * We keep a set of indexes that should
     * be loaded for lazy loading.
     * @param index:, 应该加载的儿子
     */
    function addIndexToLoaded(index) {
        if (!isValidNextIndex(index))
            return;
        let indexes = index;
        // allow the user to customize which indexes to load
        if (onSetLazy) {
            indexes = onSetLazy(index);
        }
        const indexesArray = Array.isArray(indexes) ? indexes : [indexes];
        const next = new Set(loaded);
        //重新洗牌 next= 旧的已经加载儿子们；
        indexesArray.forEach(i => {
            // don't set items which are already loaded or are invalid
            if (loaded.has(i) || !isValidNextIndex(index)) {
                return;
            }
            next.add(i);
        });
        setLoaded(next);
    }
    function doDragEnd(state) {
        const { active, movement: [mx, my], direction: [xDir, yDir], cancel, swipe: [swpx,], last, canceled, } = state;
        // console.log("拉动End",state,"active",active,"号序",indexRef.current,"canceled:",canceled,"快滑",swpx);
        if (!active && last && !canceled && (Math.abs(swpx) > 0 || (Math.abs(mx) >= width * 0.03 && Math.abs(mx) > Math.abs(my)))) {
            let Old = indexRef.current;
            let finalDir = xDir !== 0 ? xDir : mx > 0 ? 1 : -1;
            if (xDir === 0 && yDir !== 0)
                finalDir = 0; //允许水平方向滑动
            if (finalDir !== 0) {
                const idxNew = indexRef.current + (finalDir > 0 ? -1 : 1);
                const objIndex = (idxNew > childCount - 1) ? (childCount - 1) : idxNew < 0 ? 0 : idxNew;
                // console.log("拉动End Old",Old,"objIndex=",objIndex);
                if (objIndex === Old) {
                    cancel();
                    onNestDragEnd && onNestDragEnd([finalDir, 0]);
                }
                else {
                    //滑动未到位就该触发懒加载的准备机制。
                    addIndexToLoaded(objIndex);
                    indexRef.current = objIndex;
                    onRequestChange(objIndex);
                    cancel();
                }
            }
        }
        api.start(i => {
            if (i < indexRef.current - 1 || i > indexRef.current + 1)
                return { display: 'none' };
            const x = (i - indexRef.current) * width + (active ? mx : 0);
            const scale = active ? 1 - Math.abs(mx) / width / 2 : 1;
            return { x, scale, display: 'block' };
        });
    }
    const [anmprops, api] = useSprings(childCount, i => ({
        x: i * width,
        scale: 1,
        display: 'block',
    }));
    const bind = useGesture({
        onDragEnd: (state) => doDragEnd(state),
        onPointerMove: ({ event, ...sharedState }) => {
            if (event.pointerType !== "mouse")
                event.stopPropagation();
        },
    }, {
        drag: {
            filterTaps: true,
        },
    });
    //visibility:hidden替换display:none, 组件状态被轮播Tabs切换以后均能保留;
    React.useEffect(() => {
        loaded.add(index);
        indexRef.current = index;
        api.start(i => {
            if (i < indexRef.current - 1 || i > indexRef.current + 1)
                return { display: 'none' };
            const x = (i - indexRef.current) * width;
            const scale = 1;
            return { x, scale, display: 'block' };
        });
    }, [index, width, api, loaded]);
    return (_jsx("div", { className: "Pager", ref: containerRef, style: {
            overflow: "hidden",
            width: "100%",
            position: "relative",
            height: "100%",
            ...style,
        }, ...other, children: anmprops.map(({ x, display, scale }, i) => {
            const child = renderableChildren[i];
            //传递给子页面： ref是一个动作函数(this) ？绑定时刻会执行ref函数: 非当前轮播儿子组件的直接"aria-hidden"隐藏?。
            const sonprops = {
                key: i,
                tabIndex: index === i ? 0 : -1,
                "aria-hidden": i !== index,
            };
            //代表已经允许mount加载了: load=true的儿子才能算正常DOM。load=false儿子说明未能够mount进来。允许很长队列只需要加载一部分儿子。
            const load = !lazyLoad || index === i || loaded.has(i);
            return (_jsx(animated.div, { className: "Gesturepanel", style: {
                    display,
                    x,
                    width: "100%",
                    position: "absolute",
                    height: "100%",
                    touchAction: 'none',
                    overflowY: 'auto',
                }, ...bind(), children: _jsx(animated.div, { className: "Page_pane", style: {
                        scale,
                        touchAction: 'none',
                        width: "100%",
                        height: "100%",
                        // boxShadow: `0 62.5px 125px -25px rgba(50, 50, 73, 0.5), 0 37.5px 75px -37.5px rgba(0, 0, 0, 0.6)`,
                        display: "flex",
                        flexDirection: "column",
                        alignSelf: "stretch",
                        justifyContent: "flex-start",
                        flexShrink: 0,
                        overflow: "auto",
                        // outline: "none",
                        ...paneStyle
                    }, ...sonprops, children: load && child }) }, i));
        }) }, id));
});
//# sourceMappingURL=GesturePage.js.map