import { jsx as _jsx } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { useGestureResponder } from "react-gesture-responder";
import { animated, useSpring } from "@react-spring/web";
import { useMeasure } from "./Hooks/use-measure";
import { usePrevious } from "./Hooks/previous";
import { useScrollLock } from "./Hooks/use-scroll-lock";
export const Pager = React.forwardRef(({ children, id, value: index, onRequestChange, 
//focusOnChange = false,
enableScrollLock = true, enableGestures = true, enableMouse = false, lazyLoad = false, onSetLazy, animationConfig = { tension: 190, friction: 20, mass: 0.4 }, onTerminationRequest, onMoveShouldSet, minVelocity = 0.1, style, paneStyle, ...other }, ref) => {
    const containerRef = React.useRef(null); //绑定 拉拽的第一层Div;
    //拉拽滑动进行中吗?
    const [isDragging, setIsDragging] = React.useState(false);
    //已经加载的部分： 多个儿子，不一定都加载mount完毕; 轮播切换不会卸载。 [index]默认只能加载一个;
    const [loaded, setLoaded] = React.useState(() => new Set(onSetLazy ? onSetLazy(index) : [index]));
    //旧版本 const bounds = useMeasure(ref); 新的用 const { bounds } = useMeasure(ref);
    const { bounds } = useMeasure(containerRef);
    const { width } = bounds;
    //当前已经加载的儿子映射DOM;
    const childrenRefs = React.useRef(new Map());
    //上一次位置
    const previousIndex = usePrevious(index);
    //当前应该轮播位置
    const shouldFocusRef = React.useRef(index || null);
    useScrollLock(isDragging && enableScrollLock);
    React.useEffect(() => {
        if (typeof previousIndex === "number" && previousIndex !== index) {
            shouldFocusRef.current = index;
        }
        else {
            shouldFocusRef.current = null;
        }
    }, [previousIndex, index]);
    function focusByIndex(i) {
        const el = childrenRefs.current.get(i);
        if (el) {
            el.focus();
        }
    }
    // expose an imperative focus function which focuses the currently active index
    //上级组件可能使用本接口直接操作 focusByIndex切换到某一个儿子， 跳过轮播滑动:也是可能切换轮播台的。
    React.useImperativeHandle(ref, () => ({
        focus: (i) => {
            focusByIndex(i || index);
        }
    }));
    //轮播台儿子全部放到第三层，第一层拉拽绑定div，第二层是useSpring动画; 这里是 x :是水平方向的动画。
    const [{ x: nowPosX }, setSpring] = useSpring(() => ({
        x: index * -100,
        config: animationConfig
    }));
    /** 切换之后修正回调: 给 setSpring（{ ， onRest() }）用的。
     * Potentially autofocus after our animation
     */
    function onRest() {
        if (typeof shouldFocusRef.current === "number") {
            focusByIndex(shouldFocusRef.current);
        }
    }
    //每一个直接儿子都是 []；
    const renderableChildren = children.filter(child => child !== null);
    // gesture view counts
    const childCount = renderableChildren.length;
    const maxIndex = childCount - 1; //最大儿子的序号。
    const minIndex = 0; //固定的
    /**
     * Prevent invalid indexes
     */
    function isValidNextIndex(index) {
        return index >= minIndex && index <= maxIndex;
    }
    /** 跟踪设置应该加载的儿子;
     * We keep a set of indexes that should
     * be loaded for lazy loading.
     * @param index:, 应该加载的儿子
     */
    function addIndexToLoaded(index) {
        if (!isValidNextIndex(index)) {
            return;
        }
        let indexes = index;
        // allow the user to customize which indexes to load
        if (onSetLazy) {
            indexes = onSetLazy(index);
        }
        const indexesArray = Array.isArray(indexes) ? indexes : [indexes];
        const next = new Set(loaded);
        //重新洗牌 next= 旧的已经加载儿子们；
        indexesArray.forEach(i => {
            // don't set items which are already loaded or are invalid
            if (loaded.has(i) || !isValidNextIndex(index)) {
                return;
            }
            next.add(i);
        });
        setLoaded(next);
    }
    // animate into position if our index changes
    React.useEffect(() => {
        setSpring({
            x: index * -100,
            onRest
        });
        loaded.add(index);
    }, [index]);
    /** 轮播切换最终应答：执行上级组件提供的回调 onRequestChange，实际就是准备真正改变Pager本组件的输入传入参数的index：【绕了一大圈】;
     * Handle gesture end event (either touchend
     * or pan responder termination).
     */
    function releaseToPosition(x) {
        // if it's over 50% in either direction, move to that index.
        // otherwise, snap back to existing index.
        const threshold = width / 2;
        if (Math.abs(x) > threshold) {
            if (x < 0 && index < maxIndex) {
                onRequestChange(index + 1);
            }
            else if (x > 0 && index > minIndex) {
                onRequestChange(index - 1);
            }
            else {
                //右滑动到了最大，或者左边拉拽到最小的儿子。
                setSpring({ x: index * -100 });
            }
        }
        else {
            // return back! 拉拽宽度不及一半宽度的:
            setSpring({ x: index * -100, onRest });
        }
    }
    function onTermination({ delta }) {
        setIsDragging(false);
        releaseToPosition(delta[0]);
    }
    /**竟然要求速度：手速太慢了也不能切换轮播台？
    * */
    function onEnd({ delta, velocity, direction }) {
        //console.log("onEnd进delta=",　delta,"velocity",velocity,"direction",direction);
        const [x] = delta;
        setIsDragging(false);
        //X轴方向的拉拽
        if (Math.abs(direction[0]) > Math.abs(direction[1])) {
            // 1. If the force is great enough, switch to the previous index
            if (velocity > minVelocity && direction[0] > 0 && index > minIndex) {
                return onRequestChange(index - 1);
            }
            // or the next index, depending on direction
            if (velocity > minVelocity && direction[0] < 0 && index < maxIndex) {
                return onRequestChange(index + 1);
            }
        }
        releaseToPosition(x);
    }
    /**
     * Observe our pan-responder to enable gestures
     */
    const { bind } = useGestureResponder({
        onTerminationRequest,
        onStartShouldSet: () => {
            if (!enableGestures) {
                return false;
            }
            //按下开始： 不着急的
            return false;
        },
        onMoveShouldSet: (state, e) => {
            const { initial, xy, initialDirection } = state;
            if (!enableGestures) {
                return false;
            }
            const needSet = initialDirection[0] != 0;
            // allow the user to tap into this component to potentially
            // override it
            if (onMoveShouldSet) {
                return onMoveShouldSet(state, e, needSet);
            }
            //是否进入下一阶段 ==>去执行下面的onGrant()过程。
            return needSet;
        },
        onGrant: () => {
            setIsDragging(true);
        },
        onMove: ({ delta, direction }, e) => {
            //回调类型是 onMove:(state: StateType, e: ResponderEvent) => void;
            const [x] = delta;
            //第二层的动画Div实际水平方向宽度是很多个儿子拼凑的：X轴上面是 并排布局的第三层Div; 动画就是移动下水平X轴的位置了。
            const xPos = (x / width) * 100 + index * -100;
            //滑动的 立刻跟随动画效果：
            setSpring({
                x: xPos,
                immediate: true,
                onRest: () => { }
            });
            // lazy load the item we are swiping towards 滑动未到位就该触发懒加载的准备机制。 而direction[0] > 0表示水平方向的拉拽。
            addIndexToLoaded(direction[0] > 0 ? index - 1 : index + 1);
        },
        onRelease: onEnd,
        onTerminate: onTermination
    }, {
        uid: id,
        enableMouse
    });
    //拖拽触发的bind直接绑定第一层Div上了; 第二层直接flex布局x水平方向的从左到右的排放:第三层儿子们的Div宽度都是一样的。
    return (_jsx("div", { ...bind, ref: containerRef, className: "Gesture-view", style: {
            display: "flex",
            flexDirection: "column",
            overflow: "hidden",
            width: "100%",
            ...style
        }, ...other, children: _jsx(animated.div, { className: "Gesture-view__animated-container", style: {
                flexDirection: "row",
                direction: "ltr",
                willChange: "transform",
                minHeight: 0,
                flex: 1,
                display: "flex",
                transform: nowPosX.to(x => `translateX(${taper(x, maxIndex * -100)}%)`)
            }, children: renderableChildren.map((child, i) => {
                const styles = {
                    display: "flex",
                    flexDirection: "column",
                    width: "100%",
                    alignSelf: "stretch",
                    justifyContent: "flex-start",
                    flexShrink: 0,
                    height: "100%",
                    overflow: "auto",
                    outline: "none",
                    ...paneStyle
                };
                //style: styles,    { flex: `1 1 ${flexClPp[suitableCls-1]}%` },
                //传递给子页面： ref是一个动作函数(this) ？绑定时刻会执行ref函数: 非当前轮播儿子组件的直接"aria-hidden"隐藏?。
                const props = {
                    key: i,
                    tabIndex: index === i ? 0 : -1,
                    style: styles,
                    "aria-hidden": i !== index,
                    ref: (el) => {
                        childrenRefs.current.set(i, el);
                    }
                };
                //代表已经允许mount加载了: load=true的儿子才能算正常DOM。load=false儿子说明未能够mount进来。允许很长队列只需要加载一部分儿子。
                const load = !lazyLoad || index === i || loaded.has(i);
                if (typeof child === "function") {
                    return child(props, index === i, load);
                }
                return (_jsx("div", { className: "Gesture-view__pane", ...props, children: load && child }));
            }) }) }));
});
//export default React.forwardRef(Pager);
/** translateX 正值向右移动、负值向左移动。 +- 百分比相对Div
 * 返回是 百分比 [negMaxRowPercent ,0]
 * 【关键】以来flex display ,row : x轴方向顺序摆放布局多个儿子，每次切换 X 轴位移显示不同儿子。
 * Add some resistance when swiping in a direction
 * that doesn't contain another pane
 * negMaxRowPercent == - 100 * maxIndex; 最大儿子的序号 maxIndex = childCount - 1;
 */
function taper(x, negMaxRowPercent) {
    //这里折损系数 免去拉的太远了
    if (x > 0) {
        return x * 0.2;
    }
    if (x < negMaxRowPercent) {
        const diff = x - negMaxRowPercent;
        return x - diff * 0.8;
    }
    return x;
}
//# sourceMappingURL=Pager.js.map