import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import { css } from "@emotion/react";
import * as React from "react";
import { Layer } from "./Layer";
import PropTypes from "prop-types";
import { useTheme } from "./Theme/Providers";
//没有对外输出TableSectionContext的；无法import {TableSectionContext} from "customize-easy-ui-component";
const TableSectionContext = React.createContext({
    type: "TableHead"
});
//旧版本的Table才需要使用到的 context变量； 新版本已经改成参数适应;
const TableContext = React.createContext({ fixed: false });
//等待删除 代码
//@Deprecated
const TableOld = ({ children, minWidth, fixed, ...other }) => {
    const theme = useTheme();
    return (_jsx("div", { css: {
            width: "100%",
            display: "block",
            overflowX: minWidth ? "auto" : "initial"
        }, children: _jsxs("table", { css: {
                borderSpacing: 0,
                borderCollapse: "separate",
                width: "100%",
                fontFamily: theme.fonts.base,
                WebkitAppearance: "none",
                WebkitFontSmoothing: "antialiased",
                display: "table",
                minWidth,
                tableLayout: fixed ? "fixed" : undefined
            }, ...other, children: [fixed && (_jsx("colgroup", { children: fixed.map((width, i) => {
                        return _jsx("col", { width: width }, i);
                    }) })), _jsx(TableContext.Provider, { value: { fixed: fixed ? true : false }, children: children })] }) }));
};
/**
支持打印自适应，支持手机自适应。
 */
export const Table = ({ children, minWidth, fixed, printColWidth, ...other }) => {
    const theme = useTheme();
    return (_jsx("div", { css: {
            width: "100%",
            display: "block",
            overflowX: minWidth ? "auto" : "initial"
        }, children: _jsxs("table", { css: {
                borderSpacing: 0,
                borderCollapse: "separate",
                width: "100%",
                fontFamily: theme.fonts.base,
                WebkitAppearance: "none",
                WebkitFontSmoothing: "antialiased",
                display: "table",
                minWidth,
                tableLayout: fixed ? "fixed" : undefined,
                "@media print": {
                    tableLayout: printColWidth ? "fixed" : fixed ? "fixed" : undefined,
                }
            }, ...other, children: [fixed && (_jsx("colgroup", { css: {
                        "@media print": {
                            display: printColWidth ? 'none' : undefined,
                        },
                    }, children: fixed.map((width, i) => {
                        return _jsx("col", { width: width }, i);
                    }) })), printColWidth && (_jsx("colgroup", { css: {
                        "@media not print": {
                            display: 'none',
                        },
                        "@media print": {
                            display: undefined,
                        },
                    }, children: printColWidth.map((width, i) => {
                        return _jsx("col", { width: width }, i);
                    }) })), children] }) }));
};
Table.propTypes = {
    minWidth: PropTypes.string,
    fixed: PropTypes.arrayOf(PropTypes.string),
    printColWidth: PropTypes.arrayOf(PropTypes.string),
    children: PropTypes.node
};
export function TableHead({ children, ...other }) {
    return (_jsx("thead", { css: {
            display: "table-header-group"
        }, ...other, children: _jsx(TableSectionContext.Provider, { value: { type: "TableHead" }, children: children }) }));
}
export const TableRow = ({ onClick, children, ...other }) => {
    const theme = useTheme();
    const { type: tableSectionType } = React.useContext(TableSectionContext);
    const buttonProps = onClick
        ? {
            role: "button",
            tabIndex: 0
        }
        : {};
    return (_jsx("tr", { onClick: onClick, css: {
            height: tableSectionType === "TableHead" ? "31px" : "49px",
            display: "table-row",
            outline: "none",
            verticalAlign: "middle",
            cursor: onClick ? "pointer" : "default",
            ":hover": {
                background: onClick ? theme.colors.background.tint1 : "none"
            }
        }, ...buttonProps, ...other, children: children }));
};
TableRow.propTypes = {
    onClick: PropTypes.func,
    children: PropTypes.node
};
/**
 * Cell TableCell, used for both <td> and <th> elements.
 * textAlign: "justify" 导致字间距自动变大，感觉不规整，不如textAlign: "left"。
 */
const tableCellAlignments = {
    right: css({
        textAlign: "right",
        flexDirection: "row-reverse"
    }),
    left: css({
        textAlign: "left"
    }),
    center: css({
        textAlign: "center"
    }),
    justify: css({
        textAlign: "justify"
    })
};
/**
 * 旧版本的Cell组件，新版本改用Cell; 两个版本的样式不一样
 */
export const TableCell = ({ align = "left", variant, component, ellipsis, children, ...other }) => {
    const theme = useTheme();
    const { type: tableSectionType } = React.useContext(TableSectionContext);
    const Component = component || (tableSectionType === "TableHead" ? "th" : "td");
    const type = variant || (tableSectionType === "TableHead" ? "head" : "body");
    return (_jsx(Component, { css: [
            {
                zIndex: 4,
                position: "relative",
                borderBottom: "1px solid",
                borderColor: tableSectionType === "TableBody"
                    ? theme.colors.border.muted
                    : theme.colors.border.default,
                display: "table-cell",
                padding: `${theme.spaces.xs} ${theme.spaces.sm}`,
                [theme.mediaQueries.lg]: {
                    paddingLeft: theme.spaces.md,
                    paddingRight: theme.spaces.md
                },
                ":last-child": {
                    paddingRight: theme.spaces.md
                }
            },
            ellipsis && {
                whiteSpace: "nowrap",
                textOverflow: "ellipsis",
                overflow: "hidden"
            },
            type === "head"
                ? {
                    fontWeight: 500,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.muted
                }
                : {
                    fontWeight: 400,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.default
                },
            tableCellAlignments[align]
        ], scope: "col", ...other, children: children }));
};
TableCell.propTypes = {
    align: PropTypes.oneOf(["right", "left", "center", "justify"]),
    variant: PropTypes.oneOf(["body", "head"]),
    // component: PropTypes.element,
    ellipsis: PropTypes.bool,
    children: PropTypes.node
};
/*可以通过interface来约定react组件props，还有必要用react的propTypes吗？
2个会报类型错误？TableCell.propTypes {align: PropTypes.oneOf( ； component: PropTypes.elementType,}
 这个地方似乎是打包层次用到的。  报错？只好删除了。
 可以只用typescript进行验证，也可以只用prop-types进行验证，也可以两者混搭进行验证，这个并没用严格限制。
首先不管用typescript，还是prop-types，都只在开发模式下进行检查。 PropTypes是组件接收prop的约束。
*/
/**
 * 新版 TableCell 组件，支持手机上作表格形式的报告，支持打印。
 * 做报表用的样式, CCell, RCell类似 Cell,就差align格式;
 * 旧版本请用TableCell组件,两个版本的样式不一样
 */
export const Cell = ({ align = "left", variant, component, ellipsis, children, ...other }) => {
    const theme = useTheme();
    const { type: tableSectionType } = React.useContext(TableSectionContext);
    const Component = component || (tableSectionType === "TableHead" ? "th" : "td");
    const type = variant || (tableSectionType === "TableHead" ? "head" : "body");
    return (_jsx(Component, { css: [
            {
                zIndex: 4,
                position: "relative",
                border: "1px solid",
                display: "table-cell",
                padding: `${theme.spaces.xs} ${theme.spaces.xs}`,
            },
            ellipsis && {
                whiteSpace: "nowrap",
                textOverflow: "ellipsis",
                overflow: "hidden"
            },
            type === "head"
                ? {
                    fontWeight: 500,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.muted
                }
                : {
                    fontWeight: 400,
                    fontSize: theme.fontSizes[0],
                    color: theme.colors.text.default
                },
            tableCellAlignments[align]
        ], scope: "col", ...other, children: children }));
};
export function TableBody({ children, ...other }) {
    return (_jsx("tbody", { css: {
            display: "table-row-group"
        }, ...other, children: _jsx(TableSectionContext.Provider, { value: { type: "TableBody" }, children: children }) }));
}
export const ExpandingRow = ({ content, children }) => {
    const theme = useTheme();
    const [selected, setSelected] = React.useState(false);
    function close() {
        setSelected(false);
    }
    function open() {
        setSelected(true);
    }
    return (_jsxs(TableBody, { children: [_jsx(TableRow, { onClick: open, children: children }), selected && (_jsx("tr", { css: { display: "table-row", height: "100px" }, children: _jsxs("td", { colSpan: React.Children.count(children), css: {
                        borderBottom: 0,
                        width: "inherit",
                        padding: 0,
                        position: "relative"
                    }, children: [_jsx("div", { css: {
                                zIndex: 3,
                                width: "inherit",
                                position: "relative",
                                paddingBottom: "24px"
                            }, children: _jsx("div", { css: {
                                    position: "relative",
                                    whiteSpace: "normal",
                                    height: "auto",
                                    display: "block",
                                    paddingTop: "24px"
                                }, children: typeof content === "function" ? content(close) : content }) }), _jsx(Layer, { css: {
                                position: "absolute",
                                top: "-49px",
                                left: "-16px",
                                right: "-16px",
                                borderRadius: theme.radii.md,
                                bottom: 0,
                                zIndex: 2
                            }, children: null })] }) }))] }));
};
//被删除的部分 xxx.propTypes = {  };  是和rollup.js相关，一个模块打包工具/发布工具库，使用ES6的模块标准。
ExpandingRow.propTypes = {
    //content: PropTypes.oneOfType([PropTypes.func, PropTypes.node]),
    children: PropTypes.node
};
/**
大量重复的组件标签，简化: CCell代表居中显示的Cell
 */
export const CCell = ({ align = "center", variant, component, ellipsis, children, ...other }) => {
    return (Cell({
        align,
        variant,
        component,
        ellipsis,
        children,
        ...other
    }));
};
/**做点简化，免得看起来是一大堆重复性质代码：
 * 代表靠右边 显示的Cell
 */
export const RCell = ({ align = "right", variant, component, ellipsis, children, ...other }) => {
    return (Cell({
        align,
        variant,
        component,
        ellipsis,
        children,
        ...other
    }));
};
//# sourceMappingURL=Table.js.map