import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
import * as React from "react";
import { animate, motion, useMotionValue } from "framer-motion";
import { MoveFocusInside } from 'react-focus-lock';
import { PanelEnlargeCtx, useMMenuBarOcup } from "./MainMenuBar";
import { DdMenu, DdMenuItem } from "./DropdownMenu";
import { IconChevronsLeft } from "./Icons";
import { MenuItem } from "./Menu";
const containerStyle = {
    position: 'absolute',
    width: "100%",
    height: "100%",
    display: "inline-block",
    // display: "flex",
    flexDirection: 'row',
    justifyContent: 'space-between',
    flexWrap: 'nowrap',
};
const pageStyle = {
    top: 0,
    flex: "none",
    overflowX: "hidden",
};
const transition = {
    type: "spring",
    bounce: 0,
};
// const Contaier = React.forwardRef<
//     HTMLDivElement,
//     { children: React.ReactNode }
//     >((props, ref) => (
//     <div ref={ref} style={containerStyle}>
//         {props.children}
//     </div>
// ))
/**公用框架组件  Left左半列表摘要 + 右半边Console详细操作的页面。
 * 【注意】useContext(PanelEnlargeCtx);应该定义在TwoHalfFrame组件的层次顺序之下的，而不是TwoHalfFrame之前，否则PanelEnlargeCtx不正常。
 * TwoHalfFrame应该放在路由器每个路由顶部，外部加个PanelEnlargeCtx.Provider配套控制本实例TwoHalfFrame。
 * 本组件是框架，要配合 <TwoHalfRightPanel/>组件 以及<PanelEnlargeCtx.Provider 这两个东西来协同完成框架页面构建。
 * 需要在页面路由器头上加<PanelEnlargeCtx.Provider  value={{ enlarge, setEnlarge，activecas，setActivecas }}>
 * 【特别】<PanelEnlargeCtx.Provider的提供位置非常严格，可能无效:初始化取值，还是后来能修改赋值?。
 * */
export const TwoHalfFrame = React.forwardRef(({ children, rightPanel, leftStyle, rightStyle, enlarge: initEnlarge = 2, ...other }, ref) => {
    const { barHeight, bottomCss, carousel, isBigSrc, mobileList } = useMMenuBarOcup(!rightPanel);
    const [enlarge, setEnlarge] = React.useState((!rightPanel && initEnlarge === 0) ? 2 : initEnlarge);
    const containerRef = React.useRef(null);
    const x = useMotionValue(0);
    //index和对外声称的左右边页面正好相反。
    const [index, setIndex] = React.useState(0);
    /*和布局相关的：手机书评模式情况下：transformX(0)实际是显示右半边页面的。transformX(containerRef.current?.clientWidth)实际是显示左半边页面的。
    * */
    const calculateNewX = () => (!carousel ? 0 : index === 1 ? 0 : -1) * (containerRef.current?.clientWidth || 0);
    const handleEndDrag = (e, dragProps) => {
        const clientWidth = containerRef.current?.clientWidth || 0;
        const { offset, velocity } = dragProps;
        const xDrag = Math.abs(velocity.x) >= Math.abs(velocity.y);
        console.log("框剪offse", e, offset, "xDrag:", xDrag);
        if (!xDrag) {
            return animate(x, calculateNewX(), transition);
        }
        //向右边拉拽
        if (offset.x > clientWidth / 4) {
            handlePrev();
        }
        else if (offset.x < -clientWidth / 4) {
            handleNext();
        }
        else {
            animate(x, calculateNewX(), transition);
        }
    };
    //两个页面拼凑:
    const childrens = [children, rightPanel];
    const handleNext = () => {
        setIndex(index === 1 ? 0 : 1);
    };
    const handlePrev = () => {
        setIndex(index === 0 ? 1 : 0);
    };
    //外部也可以操纵，轮播台当前显示右边或左边。 【失败】注入Context实际有问题=导致ref.setActivecas()可能为undefined!
    // React.useImperativeHandle(ref, () => ({
    // }));
    const getActivecas = () => {
        return (index === 0 ? 1 : 0);
    };
    //无法直接用index对外替换Activecas？
    const setActivecas = (i) => {
        setIndex(i === 0 ? 1 : 0);
    };
    React.useEffect(() => {
        const controls = animate(x, calculateNewX(), transition);
        return controls.stop;
    }, [index]);
    //useLayoutEffect
    React.useEffect(() => {
        x.set(0);
        //若用x.set(0);在autofoucus导致自动切换显示右边页面情况，无法正常。只好改成默认set(-1*?)
        x.stop(); //这个必须加上！
        const controls = animate(x, calculateNewX(), transition);
        return controls.stop;
    }, [carousel, containerRef.current?.clientWidth, x]);
    //已经默认，不需要外边initActivecas注入
    React.useEffect(() => {
        //轮播台 右边页面=0，左边页面=1；
        setIndex(0);
    }, [rightPanel]);
    //非手机轮播台模式的左右两半边页面宽度比例： 左边没有=0，1：左边右边1比2， 2： 1比1， 3： 2比1， 4：右边没有。
    const leftpwp = enlarge === 0 ? 0 : (enlarge === 1 ? 33.4 : (enlarge === 4 ? 100 : (enlarge === 3 ? 66.6 : 50)));
    return (_jsxs(PanelEnlargeCtx.Provider, { value: { enlarge, setEnlarge,
            setActivecas: setActivecas, getActivecas: getActivecas,
        }, children: [_jsxs("div", { className: "Thframe", ref: containerRef, style: containerStyle, css: {
                    overflowX: carousel ? "hidden" : undefined,
                }, ...other, children: [_jsx(MoveFocusInside, { children: _jsx(motion.div, { className: "LeftP", style: {
                                ...pageStyle,
                                x,
                                left: `0%`,
                                // right: `${0 * 100}%`,
                                // width: "100%",
                                // height: "100vh",
                                display: "flex",
                                flexDirection: 'column',
                            }, css: {
                                height: `calc(100vh - ${barHeight})`,
                                width: (carousel || mobileList) ? "100%" : `${leftpwp}%`,
                                overflowY: 'auto',
                                ...leftStyle,
                            }, drag: carousel ? "x" : false, dragElastic: 0.3, onDragEnd: handleEndDrag, children: childrens[0] }) }), _jsx(MoveFocusInside, { children: _jsx(motion.div, { className: "RightP", style: {
                                ...pageStyle,
                                x,
                                // left: `${1 * 100}%`,
                                // right: `${1 * 100}%`,
                                // width: "100%",
                                // height: "100vh",
                            }, css: {
                                position: 'absolute',
                                left: carousel ? "100%" : mobileList ? '100%' : `${leftpwp}%`,
                                height: carousel ? `calc(100vh - ${barHeight})` : '100%',
                                width: carousel ? "100%" : mobileList ? '0%' : `${100 - leftpwp}%`,
                                ...rightStyle,
                            }, drag: carousel ? "x" : false, dragElastic: 0.3, onDragEnd: handleEndDrag, children: childrens[1] }) })] }), (!carousel && enlarge === 4) &&
                _jsx("div", { className: "Larger", css: {
                        // display: 'inline-flex',
                        right: 0,
                        bottom: 0,
                        position: 'fixed',
                    }, children: _jsxs(DdMenu, { label: "\u653E\u5927", tight: true, icon: _jsx(MenuItem, { variant: "ghost", component: "div", children: _jsx(IconChevronsLeft, {}) }), children: [_jsx(DdMenuItem, { label: "\u5DE6\u6EE1\u5C4F", onClick: (e) => { setEnlarge(4); } }), _jsx(DdMenuItem, { label: "1\u6BD42", onClick: (e) => { setEnlarge(1); } }), _jsx(DdMenuItem, { label: "1\u6BD41", onClick: (e) => { setEnlarge(2); } }), _jsx(DdMenuItem, { label: "2\u6BD41", onClick: (e) => { setEnlarge(3); } }), _jsx(DdMenuItem, { label: "\u53F3\u6EE1\u5C4F", onClick: (e) => { setEnlarge(0); } })] }) })] }));
});
//# sourceMappingURL=TwoHalfFrame.js.map